from arkindex.dataimport.models import WorkerVersion
from arkindex.documents.models import Entity, EntityType, MLClass, TranscriptionEntity
from arkindex.documents.tasks import worker_results_delete
from arkindex.project.tests import FixtureTestCase


class TestDeleteWorkerResults(FixtureTestCase):

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.text = MLClass.objects.create(name='text', corpus=cls.corpus)
        cls.version_1 = WorkerVersion.objects.get(worker__slug='reco')
        cls.version_2 = WorkerVersion.objects.get(worker__slug='dla')

        cls.vol = cls.corpus.elements.get(name='Volume 1')
        cls.classification1 = cls.vol.classifications.create(
            worker_version=cls.version_1,
            ml_class=cls.text,
            confidence=.42
        )

        cls.page1 = cls.corpus.elements.get(name='Volume 1, page 1r')
        cls.classification2 = cls.page1.classifications.create(
            worker_version=cls.version_1,
            ml_class=cls.text,
            confidence=.42
        )
        cls.transcription1 = cls.page1.transcriptions.create(
            text='something',
            worker_version=cls.version_1,
            confidence=0.42,
        )
        cls.entity = Entity.objects.create(
            type=EntityType.Person,
            corpus=cls.corpus,
            name='entity 1',
            worker_version=cls.version_2,
        )
        cls.transcription_entity1 = TranscriptionEntity.objects.create(
            transcription=cls.transcription1,
            entity=cls.entity,
            offset=2,
            length=len(cls.entity.name)
        )

        cls.page2 = cls.corpus.elements.get(name='Volume 1, page 2r')
        cls.page2.worker_version = cls.version_1
        cls.page2.save()
        cls.classification3 = cls.page2.classifications.create(
            worker_version=cls.version_2,
            ml_class=cls.text,
            confidence=.42
        )
        cls.transcription2 = cls.page2.transcriptions.create(
            text='something',
            worker_version=cls.version_1,
            confidence=0.42,
        )
        cls.transcription_entity2 = TranscriptionEntity.objects.create(
            transcription=cls.transcription2,
            entity=cls.entity,
            offset=2,
            length=len(cls.entity.name)
        )

    def test_run_on_corpus(self):
        with self.assertExactQueries('worker_results_delete_in_corpus.sql', params={
            'corpus_id': str(self.corpus.id),
            'version_id': str(self.version_1.id),
        }):
            worker_results_delete(self.corpus.id, self.version_1.id, None)

        self.corpus.refresh_from_db()

        self.assertEqual(self.corpus.elements.filter(id=self.vol.id).exists(), True)
        self.assertEqual(self.vol.classifications.filter(id=self.classification1.id).exists(), False)

        self.assertEqual(self.corpus.elements.filter(id=self.page1.id).exists(), True)
        self.page1.refresh_from_db()
        self.assertEqual(self.page1.classifications.filter(id=self.classification2.id).exists(), False)
        self.assertEqual(self.page1.transcriptions.filter(id=self.transcription1.id).exists(), False)
        self.assertEqual(TranscriptionEntity.objects.filter(id=self.transcription_entity1.id).exists(), False)
        self.assertEqual(Entity.objects.filter(id=self.entity.id).exists(), True)

        self.assertEqual(self.corpus.elements.filter(id=self.page2.id).exists(), False)

    def test_run_on_parent(self):
        with self.assertExactQueries('worker_results_delete_under_parent.sql', params={
            'corpus_id': str(self.corpus.id),
            'version_id': str(self.version_1.id),
            'parent_id': str(self.page1.id),
        }):
            worker_results_delete(self.corpus.id, self.version_1.id, self.page1.id)

        self.corpus.refresh_from_db()

        self.assertEqual(self.corpus.elements.filter(id=self.vol.id).exists(), True)
        self.assertEqual(self.vol.classifications.filter(id=self.classification1.id).exists(), True)

        self.assertEqual(self.corpus.elements.filter(id=self.page1.id).exists(), True)
        self.page1.refresh_from_db()
        self.assertEqual(self.page1.classifications.filter(id=self.classification2.id).exists(), False)
        self.assertEqual(self.page1.transcriptions.filter(id=self.transcription1.id).exists(), False)
        self.assertEqual(TranscriptionEntity.objects.filter(id=self.transcription_entity1.id).exists(), False)
        self.assertEqual(Entity.objects.filter(id=self.entity.id).exists(), True)

        self.assertEqual(self.corpus.elements.filter(id=self.page2.id).exists(), True)
        self.page2.refresh_from_db()
        self.assertEqual(self.page2.classifications.filter(id=self.classification3.id).exists(), True)
        self.assertEqual(self.page2.transcriptions.filter(id=self.transcription2.id).exists(), True)
        self.assertEqual(TranscriptionEntity.objects.filter(id=self.transcription_entity2.id).exists(), True)
        self.assertEqual(Entity.objects.filter(id=self.entity.id).exists(), True)

    def test_run_on_parent_delete_element(self):
        with self.assertExactQueries('worker_results_delete_under_parent_included.sql', params={
            'corpus_id': str(self.corpus.id),
            'version_id': str(self.version_1.id),
            'parent_id': str(self.page2.id),
        }):
            worker_results_delete(self.corpus.id, self.version_1.id, self.page2.id)

        self.corpus.refresh_from_db()

        self.assertEqual(self.corpus.elements.filter(id=self.vol.id).exists(), True)
        self.assertEqual(self.vol.classifications.filter(id=self.classification1.id).exists(), True)

        self.assertEqual(self.corpus.elements.filter(id=self.page1.id).exists(), True)
        self.page1.refresh_from_db()
        self.assertEqual(self.page1.classifications.filter(id=self.classification2.id).exists(), True)
        self.assertEqual(self.page1.transcriptions.filter(id=self.transcription1.id).exists(), True)
        self.assertEqual(TranscriptionEntity.objects.filter(id=self.transcription_entity1.id).exists(), True)
        self.assertEqual(Entity.objects.filter(id=self.entity.id).exists(), True)

        self.assertEqual(self.corpus.elements.filter(id=self.page2.id).exists(), False)

    def test_run_all_versions(self):
        with self.assertExactQueries('worker_results_delete_all_versions.sql', params={
            'corpus_id': str(self.corpus.id),
        }):
            worker_results_delete(self.corpus.id, None, None)

        self.corpus.refresh_from_db()

        self.assertEqual(self.corpus.elements.filter(id=self.vol.id).exists(), True)
        self.assertEqual(self.vol.classifications.filter(id=self.classification1.id).exists(), False)

        self.assertEqual(self.corpus.elements.filter(id=self.page1.id).exists(), True)
        self.page1.refresh_from_db()
        self.assertEqual(self.page1.classifications.filter(id=self.classification2.id).exists(), False)
        self.assertEqual(self.page1.transcriptions.filter(id=self.transcription1.id).exists(), False)
        self.assertEqual(TranscriptionEntity.objects.filter(id=self.transcription_entity1.id).exists(), False)
        self.assertEqual(Entity.objects.filter(id=self.entity.id).exists(), True)

        self.assertEqual(self.corpus.elements.filter(id=self.page2.id).exists(), False)
